import shutil
import unittest

from approvaltests import get_default_namer, verify, Options, verify_file, approvals
from approvaltests.file_approver import FileApprover
from approvaltests.reporters.generic_diff_reporter_factory import (
    GenericDiffReporterFactory,
)
from approvaltests.reporters.testing_reporter import ReporterForTesting
from approvaltests.string_writer import StringWriter


class FileApproverTests(unittest.TestCase):
    @staticmethod
    def test_compare_same_files():
        writer = StringWriter("a")
        writer.write_received_file("a.txt")
        shutil.copy("a.txt", "a_same.txt")
        FileApprover.verify_files("a.txt", "a_same.txt", None, Options().comparator)

    def test_compare_different_files(self):
        reporter = ReporterForTesting()
        FileApprover.verify_files("a.txt", "b.txt", reporter, Options().comparator)
        self.assertTrue(reporter.called)

    def test_scrubbed_files(self):
        verify_file("a.txt", options=Options().with_scrubber(lambda t: "<scrubbed>"))

    def test_full(self):
        namer = get_default_namer()
        writer = StringWriter("b")
        reporter = ReporterForTesting()
        FileApprover.verify(namer, writer, reporter, Options().comparator)
        self.assertTrue(reporter.called)

    def test_returns_error_when_files_are_different(self):
        approvals.settings().allow_multiple_verify_calls_for_this_method()
        namer = get_default_namer()
        writer = StringWriter("b")
        reporter = ReporterForTesting()
        error = FileApprover.verify(namer, writer, reporter, Options().comparator)
        import re

        replaced = re.sub("ved: .*approved_files.", "ved: <rootdir>/", error)

        verify(replaced)

    def test_returns_none_when_files_are_same_files(self):
        namer = get_default_namer()
        writer = StringWriter("b")
        reporter = GenericDiffReporterFactory().get_first_working()
        error = FileApprover.verify(namer, writer, reporter, Options().comparator)
        self.assertEqual(None, error)


if __name__ == "__main__":
    unittest.main()



class SplitCode:
    def __init__(self, before_method, after_method, tab):
        self.before_method = before_method
        self.after_method = after_method
        self.tab = tab

    def __str__(self):
        return f"before:\n{self.before_method}\nafter:\n{self.after_method}"

    @staticmethod
    def on_method(code, method_name) -> "SplitCode":
        lines = code.split("\n")
        before = []
        after = []
        inside_method = False
        inside_doc_string = False
        tab = "    "
        after_method = False
        state = 0

        for line in lines:
            stripped_line = line.strip()

            if state == 0:
                before.append(line)
            if stripped_line.startswith(f"def {method_name}("):
                state = 1
                continue
            if state == 1:
                tab = line[: line.find(stripped_line)]
                if stripped_line.startswith('"""'):
                    state = 2
                    continue
                else:
                    state = 3
            if state == 2:
                if stripped_line.startswith('"""'):
                    state = 3
                continue
            if state == 3:
                after.append(line)

        return SplitCode("\n".join(before), "\n".join(after), tab)

    def indent(self, received_text):
        lines = received_text.split("\n")
        indented_lines = [f"{self.tab}{line}" for line in lines]
        return "\n".join(indented_lines)



from approval_utilities.utilities.multiline_string_utils import remove_indentation_from
from approvaltests import verify
from approvaltests.inline.split_code import SplitCode


def test_splitting_code():
    code = remove_indentation_from(
        '''
        def other_code():
           pass
        def testy_mctest():
            """
            Approved: test_inline_approvals.py
            Received:test_inline_approvals.recieved.txt
            """
            verify(greeting(), options = Options().inline())
            
        def greeting():
            # start of greeting() method
            return "hello world"
        '''
    )
    verify(SplitCode.on_method(code, "testy_mctest"))



import tempfile
from inspect import FrameInfo
from pathlib import Path
from typing import Optional, Callable, Any

from approval_utilities.utilities.multiline_string_utils import remove_indentation_from
from approvaltests import Namer, StackFrameNamer
from approvaltests.namer.inline_python_reporter import InlinePythonReporter


class InlineComparator(Namer):
    def get_approved_filename(self, base: Optional[str] = None) -> str:
        file = tempfile.NamedTemporaryFile(suffix=".approved.txt", delete=False).name
        docs = self.get_test_method_doc_string()
        Path(file).write_text(docs)
        return file

    def get_received_filename(self, base: Optional[str] = None) -> str:
        return tempfile.NamedTemporaryFile(suffix=".received.txt", delete=False).name

    def get_test_method_doc_string(self):
        test_stack_frame: FrameInfo = StackFrameNamer.get_test_frame()
        method: Callable[..., Any] = self.get_caller_method(test_stack_frame)
        return remove_indentation_from(method.__doc__)

    def get_caller_method(self, caller_frame) -> Callable:
        caller_function_name: str = caller_frame[3]
        caller_function_object = caller_frame.frame.f_globals.get(
            caller_function_name, None
        )
        return caller_function_object

    def register(self, options: "Options", show_code: bool):
        options2 = options.with_namer(self)
        if show_code:
            options2 = options2.with_reporter(InlinePythonReporter())
        return options2
